/*
* Mateusz Jurczak
* C++20 Modules
* 
* Moduly to funkcjonalnosc jezyka C++, ktora stanowi obecnie alternatywe dla
* niektorych przypadkow uzycia plikow naglowkowych. Modul to zestaw plikow
* zrodlowych, ktore sa kompilowane niezaleznie od jednostek tlumaczeniowych
* ktore je importuja. Czesto skracaja czas kompilacji. Makra, dyrektywy
* preprocesora i nieeksportowane elementy nie sa widoczne poza modulem. Moduly
* nie maja wplywu na kompilacje jednostki tlumaczeniowej, ktora je importuje.
* 
* Jak czytamy na MSDN, aktualne wersje VisualStudio 2022 posiadaja w pelni
* zaimplementowane moduly ze standardu C++20 w swoim kompilatorze C++:
* > As of Visual Studio 2022 version 17.1, C++20 standard modules are fully
* > implemented in the Microsoft C++ compiler.
* 
* Na stronie GNU GCC czytamy jednak, ze wsprarcie kompilatora g++ dla modulow
* ze standardu C++20 jest czesciowe (wiecej w zrodle):
* > G++'s modules support is not complete.
*/

import A;
import B;
import C;

void checkA() {
	// uzycie funkcji z modulu ModuleA
	moduleAExportedFunction();

	// ponizszej funkcji nie mozna wywolac, bo nie jest wyeksportowana
	// moduleANotExportedFunction();

	// uzycie zmiennej globalnej z modulu ModuleA
	moduleAGlobalVariable = 1;

	// importowanie modulu A zaimportowalo naglowek <iostream>
	std::cout << "import <iostream>\n";

	// wywolanie funkcji z przestrzeni ns1 z modulu A
	ns1::func1();
	// ponizsza funkcja nie jest wyeksportowana
	// ns1::func2();

	// wywolanie funkcji z wyeksportowanej przestrzeni nazw
	ns2::func1();
	ns2::func2();
}

void checkB() {
	// wywolanie wszystkich funkcji i konstruktorow z podzielonego modulu B
	printEntry(Entry(Name("John", "Deer"), 30, 'M'));
}

void checkC() {
	// wywolanie funkcji z wydzielonym interfejsem i implementacja
	std::cout << getRandomDice() << "\n";
}

int main() {
	// sprawdzenie podstawowych funkcjonalnosci modulow
	checkA();
	// szersze spojrzenie na modulu, jego elementy i partycje (czesci)
	checkB();
	// podzial plikow na jednostki interfejsu i implementacji
	checkC();
}

/*
Zrodla:
https://en.cppreference.com/w/cpp/language/modules
https://learn.microsoft.com/en-us/cpp/cpp/modules-cpp
https://gcc.gnu.org/onlinedocs/gcc/C_002b_002b-Modules.html
https://gcc.gnu.org/onlinedocs/gcc/gcc-command-options/c%2B%2B-modules.html
https://www.open-std.org/jtc1/sc22/wg21/docs/papers/2019/p1103r3.pdf
*/
